@@ -32,6 +32,7 @@ using Mono.Unix;
 
 using Banshee.Base;
 using Banshee.MediaEngine;
+using Banshee.Gstreamer;
 
 namespace Banshee.MediaEngine.Gstreamer
 {
@@ -40,6 +41,7 @@ namespace Banshee.MediaEngine.Gstreamer
     internal delegate void GstPlaybackErrorCallback(IntPtr engine, IntPtr error, IntPtr debug);
     internal delegate void GstPlaybackStateChangedCallback(IntPtr engine, int old_state, int new_state, int pending_state);
     internal delegate void GstPlaybackIterateCallback(IntPtr engine);
+    internal delegate void GstPlaybackBufferingCallback(IntPtr engine, int buffering_progress);
 
     public class GstreamerPlayerEngine : PlayerEngine
     {
@@ -49,6 +51,10 @@ namespace Banshee.MediaEngine.Gstreamer
         private GstPlaybackErrorCallback error_callback;
         private GstPlaybackStateChangedCallback state_changed_callback;
         private GstPlaybackIterateCallback iterate_callback;
+        private GstPlaybackBufferingCallback buffering_callback;
+        private GstTaggerTagFoundCallback tag_found_callback;
+        
+        private bool buffering_finished;
         
         public GstreamerPlayerEngine()
         {
@@ -64,11 +70,15 @@ namespace Banshee.MediaEngine.Gstreamer
             error_callback = new GstPlaybackErrorCallback(OnError);
             state_changed_callback = new GstPlaybackStateChangedCallback(OnStateChanged);
             iterate_callback = new GstPlaybackIterateCallback(OnIterate);
+            buffering_callback = new GstPlaybackBufferingCallback(OnBuffering);
+            tag_found_callback = new GstTaggerTagFoundCallback(OnTagFound);
             
             gst_playback_set_eos_callback(handle, eos_callback);
             gst_playback_set_iterate_callback(handle, iterate_callback);
             gst_playback_set_error_callback(handle, error_callback);
             gst_playback_set_state_changed_callback(handle, state_changed_callback);
+            gst_playback_set_buffering_callback(handle, buffering_callback);
+            gst_playback_set_tag_found_callback(handle, tag_found_callback);
         }
         
         public override void Dispose()
@@ -101,15 +111,18 @@ namespace Banshee.MediaEngine.Gstreamer
             gst_playback_pause(handle);
             OnStateChanged(PlayerEngineState.Paused);
         }
-        
-        public override void SetProperty(string key, object value)
+
+        public override IntPtr [] GetBaseElements()
         {
-            IntPtr val_ptr = (IntPtr)value;
-            IntPtr key_ptr = GLib.Marshaller.StringToPtrGStrdup(key);
-            gst_playback_playbin_set_property(handle, key_ptr, val_ptr);
-            GLib.Marshaller.Free(key_ptr);
+            IntPtr [] elements = new IntPtr[3];
+            
+            if(gst_playback_get_pipeline_elements(handle, out elements[0], out elements[1], out elements[2])) {
+                return elements;
+            }
+            
+            return null;
         }
-        
+
         private void OnEos(IntPtr engine)
         {
             Close();
@@ -141,6 +154,21 @@ namespace Banshee.MediaEngine.Gstreamer
         {
         }
         
+        private void OnBuffering(IntPtr engine, int progress)
+        {
+            if(buffering_finished && progress >= 100) {
+                return;
+            }
+            
+            buffering_finished = progress >= 100;
+            OnEventChanged(PlayerEngineEvent.Buffering, Catalog.GetString("Buffering"), (double)progress / 100.0);
+        }
+        
+        private void OnTagFound(string tagName, ref GLib.Value value, IntPtr userData)
+        {
+            OnTagFound(GstTagger.ProcessNativeTagResult(tagName, ref value));
+        }
+        
         public override ushort Volume {
             get { return (ushort)gst_playback_get_volume(handle); }
             set { 
@@ -157,6 +185,10 @@ namespace Banshee.MediaEngine.Gstreamer
             }
         }
         
+        public override bool CanSeek {
+            get { return gst_playback_can_seek(handle); }
+        }
+        
         public override uint Length {
             get { return (uint)gst_playback_get_duration(handle) / 1000; }
         }
@@ -193,6 +225,12 @@ namespace Banshee.MediaEngine.Gstreamer
         [DllImport("libbanshee")]
         private static extern void gst_playback_set_iterate_callback(HandleRef engine, GstPlaybackIterateCallback cb);
         
+        [DllImport("libbanshee")]
+        private static extern void gst_playback_set_buffering_callback(HandleRef engine, GstPlaybackBufferingCallback cb);
+        
+        [DllImport("libbanshee")]
+        private static extern void gst_playback_set_tag_found_callback(HandleRef engine, GstTaggerTagFoundCallback cb);
+        
         [DllImport("libbanshee")]
         private static extern void gst_playback_open(HandleRef engine, IntPtr uri);
         
@@ -211,6 +249,9 @@ namespace Banshee.MediaEngine.Gstreamer
         [DllImport("libbanshee")]
         private static extern int gst_playback_get_volume(HandleRef engine);
         
+        [DllImport("libbanshee")]
+        private static extern bool gst_playback_can_seek(HandleRef engine);
+        
         [DllImport("libbanshee")]
         private static extern void gst_playback_set_position(HandleRef engine, ulong time_ms);
         
@@ -221,7 +262,8 @@ namespace Banshee.MediaEngine.Gstreamer
         private static extern ulong gst_playback_get_duration(HandleRef engine);
         
         [DllImport("libbanshee")]
-        private static extern void gst_playback_playbin_set_property(HandleRef engine, IntPtr key, IntPtr value);
+        private static extern bool gst_playback_get_pipeline_elements(HandleRef engine, out IntPtr playbin,
+            out IntPtr audiobin, out IntPtr audiotee);    
     }
     
     
