@@ -54,8 +54,6 @@ namespace Banshee
         private Glade.XML gxml;
 
         [Widget] private Gtk.Window WindowPlayer;
-        [Widget] private Gtk.HScale ScaleTime;
-        [Widget] private Gtk.Label LabelInfo;
         [Widget] private HPaned SourceSplitter;
         [Widget] private Button HeaderCycleButton;
 
@@ -82,11 +80,12 @@ namespace Banshee
         private ActionButton burn_button;
         private ActionButton rip_button;
         
+        private SeekSlider seek_slider;
+        private StreamPositionLabel stream_position_label;
+        
         private ActionButton sync_dap_button;
         [Widget] private ProgressBar dapDiskUsageBar;
         
-        private HighlightMessageArea audiocd_statusbar;
-        
         private bool incrementedCurrentSongPlayCount;
     
         public Gtk.Window Window {
@@ -146,12 +145,7 @@ namespace Banshee
             
             PlayerEngineCore.EventChanged += OnPlayerEngineEventChanged;
             PlayerEngineCore.StateChanged += OnPlayerEngineStateChanged;
-            
-            if(Globals.AudioCdCore != null) {
-                Globals.AudioCdCore.DiskRemoved += OnAudioCdCoreDiskRemoved;
-                Globals.AudioCdCore.Updated += OnAudioCdCoreUpdated;
-            }
-            
+
             DapCore.DapAdded += OnDapCoreDeviceAdded;
             LogCore.Instance.Updated += OnLogCoreUpdated;
             ImportManager.Instance.ImportRequested += OnImportManagerImportRequested;
@@ -183,6 +177,7 @@ namespace Banshee
             }
             
             WindowPlayer.AddAccelGroup(Globals.ActionManager.UI.AccelGroup);
+            Globals.ActionManager.PlaybackSeekActions.Sensitive = false;
             
             LoadSettings();
             
@@ -256,6 +251,17 @@ namespace Banshee
             // Main Menu
             (gxml["MainMenuContainer"] as Container).Add(Globals.ActionManager.GetWidget("/MainMenu"));
       
+            // Seek Slider/Position Label
+            seek_slider = new SeekSlider();
+            seek_slider.SetSizeRequest(125, -1);
+            seek_slider.SeekRequested += OnSeekRequested;
+            
+            stream_position_label = new StreamPositionLabel(seek_slider);
+            
+            (gxml["SeekContainer"] as Box).PackStart(seek_slider, false, false, 0);
+            (gxml["SeekContainer"] as Box).PackStart(stream_position_label, false, false, 0);
+            gxml["SeekContainer"].ShowAll();
+      
             // Playback Buttons
             HBox playback_box = gxml["LeftToolbarContainer"] as HBox;
             
@@ -300,13 +306,6 @@ namespace Banshee
             volumeButton.VolumeChanged += OnVolumeScaleChanged;
             
             // Footer 
-            audiocd_statusbar = new HighlightMessageArea();
-            audiocd_statusbar.BorderWidth = 5;
-            audiocd_statusbar.LeftPadding = 15;
-            audiocd_statusbar.ButtonClicked += OnAudioCdStatusBarButtonClicked;
-            
-            (gxml["MainContainer"] as Box).PackStart(audiocd_statusbar, false, false, 0);
-            
             LabelStatusBar = new Label(Catalog.GetString("Banshee Music Player"));
             LabelStatusBar.Show();
             
@@ -391,6 +390,7 @@ namespace Banshee
             playlistView = new PlaylistView(playlistModel);
             InterfaceElements.PlaylistView = playlistView;
             ((Gtk.ScrolledWindow)gxml["LibraryContainer"]).Add(playlistView);
+            InterfaceElements.PlaylistContainer = gxml["LibraryContainer"] as Container;
             playlistView.Show();
             playlistModel.Updated += OnPlaylistUpdated;
             playlistModel.Stopped += OnPlaylistStopped;
@@ -414,9 +414,6 @@ namespace Banshee
             
             (gxml["LeftContainer"] as VBox).PackStart(new ActiveUserEventsManager(), false, false, 0);
 
-            // Misc
-            SetInfoLabel(Catalog.GetString("Idle"));
-
             // Window Events
             WindowPlayer.KeyPressEvent += OnKeyPressEvent;
             WindowPlayer.ConfigureEvent += OnWindowPlayerConfigureEvent;
@@ -455,7 +452,7 @@ namespace Banshee
             SetTip(previous_button, Catalog.GetString("Play previous song"));
             SetTip(playpause_button, Catalog.GetString("Play/pause current song"));
             SetTip(next_button, Catalog.GetString("Play next song"));
-            SetTip(gxml["ScaleTime"], Catalog.GetString("Current position in song"));
+            SetTip(seek_slider, Catalog.GetString("Current position in song"));
             SetTip(dapDiskUsageBar, Catalog.GetString("Device disk usage"));
             SetTip(sync_dap_button, Catalog.GetString("Synchronize music library to device"));
             SetTip(volumeButton, Catalog.GetString("Adjust volume"));
@@ -644,12 +641,7 @@ namespace Banshee
             Globals.Dispose();
             Application.Quit();
         }
-      
-        private void SetInfoLabel(string text)
-        {
-            LabelInfo.Markup = "<span size=\"small\">" + GLib.Markup.EscapeText(text) + "</span>";
-        }
-        
+
         public void UpdateMetaDisplay()
         {
             TrackInfo track = PlayerEngineCore.CurrentTrack;
@@ -703,11 +695,25 @@ namespace Banshee
             Globals.Configuration.Set(GConfKeys.WindowHeight, height);
         }
         
+        private bool accel_group_active = true;
+        
         [GLib.ConnectBefore]
         private void OnKeyPressEvent(object o, KeyPressEventArgs args)
         {
             bool handled = false;
             
+            if(WindowPlayer.Focus is Entry && Gtk.Global.CurrentEvent is Gdk.EventKey) {
+                if(accel_group_active) {
+                    WindowPlayer.RemoveAccelGroup(Globals.ActionManager.UI.AccelGroup);
+                    accel_group_active = false;
+                 }
+            } else {
+                if(!accel_group_active) {
+                    WindowPlayer.AddAccelGroup(Globals.ActionManager.UI.AccelGroup);
+                    accel_group_active = true;
+                }
+            }
+            
             switch(args.Event.Key) {
                 case Gdk.Key.J:
                 case Gdk.Key.j:
@@ -719,27 +725,6 @@ namespace Banshee
                         handled = true;
                     }
                     break;
-                case Gdk.Key.Left:
-                    if((args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
-                        PlayerEngineCore.Position -= SkipDelta;
-                        handled = true;
-                    } else if((args.Event.State & Gdk.ModifierType.ShiftMask) != 0) {
-                        PlayerEngineCore.Position = 0;
-                        handled = true;
-                    }
-                    break;
-                case Gdk.Key.Right:
-                    if((args.Event.State & Gdk.ModifierType.ControlMask) != 0) {
-                        PlayerEngineCore.Position += SkipDelta;
-                        handled = true;
-                    } 
-                    break;
-                case Gdk.Key.space:
-                    if(!searchEntry.HasFocus && !sourceView.EditingRow) {
-                        PlayPause();
-                        handled = true;
-                    }
-                    break;
             }
             
             args.RetVal = handled;
@@ -796,32 +781,20 @@ namespace Banshee
             PlayerEngineCore.Volume = (ushort)volume;
             Globals.Configuration.Set(GConfKeys.Volume, volume);
         }
+        
+        private void OnSeekRequested(object o, EventArgs args)
+        {
+            PlayerEngineCore.Position = (uint)seek_slider.Value;
+        }
    
         // ---- Player Event Handlers ----
         
-        private void SetPositionLabel(long position)
-        {
-            if(PlayerEngineCore.CurrentTrack == null) {
-                return;
-            }
-            
-            SetInfoLabel(
-                // Translators: position in song. eg, "0:37 of 3:48"
-                String.Format(Catalog.GetString("{0} of {1}"),
-                          String.Format("{0}:{1:00}", position / 60, position % 60),
-                          String.Format("{0}:{1:00}", PlayerEngineCore.CurrentTrack.Duration.Minutes, 
-                          PlayerEngineCore.CurrentTrack.Duration.Seconds))
-            );    
-        }
-        
         private void OnPlayerEngineStateChanged(object o, PlayerEngineStateArgs args)
         {
             switch(args.State) {
                 case PlayerEngineState.Loaded:
                     incrementedCurrentSongPlayCount = false;
-                    ScaleTime.Adjustment.Lower = 0;
-                    ScaleTime.Adjustment.Upper = PlayerEngineCore.CurrentTrack.Duration.TotalSeconds;
-                    
+                    seek_slider.Duration = PlayerEngineCore.CurrentTrack.Duration.TotalSeconds;
                     UpdateMetaDisplay();
                     playlistView.QueueDraw();
                     
@@ -838,24 +811,21 @@ namespace Banshee
                     break;
                 case PlayerEngineState.Idle:
                     Globals.ActionManager.UpdateAction("PlayPauseAction", Catalog.GetString("Play"), "media-playback-start");
-                    ScaleTime.Adjustment.Lower = 0;
-                    ScaleTime.Adjustment.Upper = 0;
-                    ScaleTime.Value = 0;
-                    SetInfoLabel(Catalog.GetString("Idle"));
+                    seek_slider.SetIdle();
                     trackInfoHeader.SetIdle();
                     
                     UpdateMetaDisplay();
                     
                     break;
                 case PlayerEngineState.Paused:
-                    Globals.ActionManager.UpdateAction("PlayPauseAction", Catalog.GetString("Play"), 
-                        "media-playback-start");
+                    Globals.ActionManager.UpdateAction("PlayPauseAction", Catalog.GetString("Play"), "media-playback-start");
                     break;
                 case PlayerEngineState.Playing:
-                    Globals.ActionManager.UpdateAction("PlayPauseAction", Catalog.GetString("Pause"), 
-                        "media-playback-pause");
+                    Globals.ActionManager.UpdateAction("PlayPauseAction", Catalog.GetString("Pause"), "media-playback-pause");
                     break;
             }
+            
+            Globals.ActionManager.PlaybackSeekActions.Sensitive = args.State != PlayerEngineState.Idle;
         }
         
         private void OnPlayerEngineEventChanged(object o, PlayerEngineEventArgs args)
@@ -868,70 +838,55 @@ namespace Banshee
                     playlistModel.Continue();
                     playlistView.UpdateView();
                     break;
+                case PlayerEngineEvent.StartOfStream:
+                    //seek_slider.CanSeek = PlayerEngineCore.CanSeek;
+                    seek_slider.CanSeek = true;
+                    break;
                 case PlayerEngineEvent.Volume:
                     volumeButton.Volume = PlayerEngineCore.Volume;
                     break;
+                case PlayerEngineEvent.Buffering:
+                    if(args.BufferingPercent >= 1.0) {
+                        stream_position_label.IsBuffering = false;
+                        break;
+                    }
+                    
+                    stream_position_label.IsBuffering = true;
+                    stream_position_label.BufferingProgress = args.BufferingPercent;
+                    break;
+                case PlayerEngineEvent.Error:
+                    LogCore.Instance.PushError(Catalog.GetString("Playback Error"), args.Message);
+                    UpdateMetaDisplay();
+                    break;
+                case PlayerEngineEvent.TrackInfoUpdated:
+                    UpdateMetaDisplay();
+                    playlistView.QueueDraw();
+                    break;
             }
         }
-        
+
         private void OnPlayerEngineTick()
         {
+            seek_slider.CanSeek = PlayerEngineCore.CanSeek;
+            seek_slider.SeekValue = PlayerEngineCore.Position;
+            
             if(PlayerEngineCore.CurrentTrack == null) {
                 return;
             }
-             
+            
             if(PlayerEngineCore.Length > 0 && PlayerEngineCore.CurrentTrack.Duration.TotalSeconds <= 0.0) {
                 PlayerEngineCore.CurrentTrack.Duration = new TimeSpan(PlayerEngineCore.Length * TimeSpan.TicksPerSecond);
                 PlayerEngineCore.CurrentTrack.Save();
                 playlistView.QueueDraw();
-                ScaleTime.Adjustment.Upper = PlayerEngineCore.CurrentTrack.Duration.TotalSeconds;
+                seek_slider.Duration = PlayerEngineCore.CurrentTrack.Duration.TotalSeconds;
             }
-                
+            
             if(PlayerEngineCore.Length > 0 && PlayerEngineCore.Position > PlayerEngineCore.Length / 2
                 && !incrementedCurrentSongPlayCount) {
                 PlayerEngineCore.CurrentTrack.IncrementPlayCount();
                 incrementedCurrentSongPlayCount = true;
                 playlistView.QueueDraw();
             }
-                
-            if(updateEnginePosition) {
-                if(setPositionTimeoutId > 0) {
-                    GLib.Source.Remove(setPositionTimeoutId);
-                }
-                
-                setPositionTimeoutId = GLib.Timeout.Add(100, new GLib.TimeoutHandler(SetPositionTimeoutCallback));
-            
-                SetPositionLabel(PlayerEngineCore.Position);
-            }
-        }
-        
-        private bool SetPositionTimeoutCallback()
-        {
-            setPositionTimeoutId = 0;
-            ScaleTime.Value = PlayerEngineCore.Position;
-            
-            return false;
-        }
-        
-        [GLib.ConnectBeforeAttribute]
-        private void OnScaleTimeMoveSlider(object o, EventArgs args)
-        {
-            SetPositionLabel((long)ScaleTime.Value);
-        }
-        
-        [GLib.ConnectBeforeAttribute]
-        private void OnScaleTimeButtonPressEvent(object o, 
-            ButtonPressEventArgs args)
-        {
-            updateEnginePosition = false;
-        }
-        
-        [GLib.ConnectBeforeAttribute]
-        private void OnScaleTimeButtonReleaseEvent(object o, 
-            ButtonReleaseEventArgs args)
-        {
-            PlayerEngineCore.Position = (uint)ScaleTime.Value;
-            updateEnginePosition = true;
         }
         
         // ---- Playlist Event Handlers ----
@@ -969,6 +924,8 @@ namespace Banshee
             ThreadAssist.ProxyToMain(HandleSourceChanged);
         }
         
+        private uint source_update_draw_timeout = 0;
+        
         private void OnSourceManagerSourceUpdated(SourceEventArgs args)
         {
             if(args.Source == SourceManager.ActiveSource) {
@@ -976,6 +933,12 @@ namespace Banshee
                 
                 if(playlistModel.Count() == 0 && args.Source.Count > 0) {
                     playlistModel.ReloadSource();
+                } else if(source_update_draw_timeout == 0) {
+                    source_update_draw_timeout = GLib.Timeout.Add(500, delegate {
+                        playlistView.QueueDraw();
+                        source_update_draw_timeout = 0;
+                        return false;
+                    });
                 }
                 
                 gxml["SearchLabel"].Sensitive = args.Source.SearchEnabled;
@@ -1010,6 +973,8 @@ namespace Banshee
                 alignment.Remove(alignment.Child);
             }
             
+            InterfaceElements.DetachPlaylistContainer();
+            
             alignment.Add(playlist_container);
             alignment.ShowAll();
             
@@ -1029,9 +994,12 @@ namespace Banshee
                 return;
             } 
             
-            alignment.Remove(alignment.Child);
+            if(alignment.Child != null) {
+                alignment.Remove(alignment.Child);
+            }
+            
             alignment.Add(SourceManager.ActiveSource.ViewWidget);
-            alignment.ShowAll();
+            alignment.Show();
             
             gxml["PlaylistHeaderBox"].Visible = SourceManager.ActiveSource.ShowPlaylistHeader;
         }
@@ -1039,6 +1007,10 @@ namespace Banshee
         private void OnSourceTrackAdded(object o, TrackEventArgs args)
         {
             if(SourceManager.ActiveSource == o) {
+                if(searchEntry.IsQueryAvailable && !DoesTrackMatchSearch(args.Track)) {
+                    return;
+                }
+                
                 playlistModel.AddTrack(args.Track);
             }
         }
@@ -1088,68 +1060,7 @@ namespace Banshee
                 Globals.ActionManager["RenameSourceAction"].Label = Catalog.GetString("Rename Playlist");            
             }
         }
-        
-        private void OnAudioCdStatusBarButtonClicked(object o, EventArgs args)
-        {
-            AudioCdSource source = SourceManager.ActiveSource as AudioCdSource;
-            if(source == null) {
-                return;
-            }
-            
-            source.Disk.QueryMetadata();
-        }
-        
-        private void UpdateAudioCdStatus(AudioCdDisk disk)
-        {
-            string status = null;
-            Gdk.Pixbuf icon = null;
-            
-            switch(disk.Status) {
-                case AudioCdLookupStatus.ReadingDisk:
-                    status = Catalog.GetString("Reading table of contents from CD...");
-                    icon = IconThemeUtils.LoadIcon(22, "media-cdrom", "gnome-dev-cdrom-audio", "source-cd-audio");
-                    audiocd_statusbar.ShowCloseButton = false;
-                    break;
-                case AudioCdLookupStatus.SearchingMetadata:
-                    status = Catalog.GetString("Searching for CD metadata...");
-                    icon = IconThemeUtils.LoadIcon(22, "system-search", Stock.Find);
-                    audiocd_statusbar.ShowCloseButton = false;
-                    break;
-                case AudioCdLookupStatus.SearchingCoverArt:
-                    status = Catalog.GetString("Searching for CD cover art...");
-                    icon = IconThemeUtils.LoadIcon(22, "system-search", Stock.Find);
-                    audiocd_statusbar.ShowCloseButton = false;
-                    break;
-                case AudioCdLookupStatus.ErrorNoConnection:
-                    status = Catalog.GetString("Cannot search for CD metadata: " + 
-                        "there is no available Internet connection");
-                    icon = IconThemeUtils.LoadIcon(22, "network-wired", Stock.Network);
-                    audiocd_statusbar.ShowCloseButton = true;
-                    break;
-                case AudioCdLookupStatus.ErrorLookup:
-                    status = Catalog.GetString("Could not fetch metadata for CD.");
-                    icon = IconThemeUtils.LoadIcon(22, Stock.DialogError);
-                    audiocd_statusbar.ShowCloseButton = true;
-                    break;
-                case AudioCdLookupStatus.Success:
-                default:
-                    status = null;
-                    icon = null;
-                    break;
-            }
-            
-            if(disk.Status == AudioCdLookupStatus.ErrorLookup) {
-                audiocd_statusbar.ButtonLabel = Stock.Refresh;
-                audiocd_statusbar.ButtonUseStock = true;
-            } else {
-                audiocd_statusbar.ButtonLabel = null;
-            }
-            
-            audiocd_statusbar.Visible = status != null;
-            audiocd_statusbar.Message = String.Format("<big>{0}</big>", GLib.Markup.EscapeText(status));
-            audiocd_statusbar.Pixbuf = icon;
-        }
-        
+     
         // Called when SourceManager emits an ActiveSourceChanged event.
         private void HandleSourceChanged(object o, EventArgs args)
         {
@@ -1159,14 +1070,10 @@ namespace Banshee
             }
 
             searchEntry.CancelSearch();
-            audiocd_statusbar.Visible = false;
             
             if(source is DapSource) {
                 DapSource dap_source = source as DapSource;
                 UpdateDapDiskUsageBar(dap_source);
-            } else if(source is AudioCdSource) {
-                AudioCdSource cdSource = source as AudioCdSource;
-                UpdateAudioCdStatus(cdSource.Disk);
             }
             
             UpdateViewName(source);   // Bold label below track info
@@ -1230,43 +1137,6 @@ namespace Banshee
             args.Dap.SaveFinished += OnDapSaveFinished;
         }
         
-        private void OnAudioCdCoreDiskRemoved(object o, AudioCdCoreDiskRemovedArgs args)
-        {
-            Source source = SourceManager.ActiveSource;
-            if(source == null) {
-                return;
-            }
-
-            if(source is AudioCdSource) {
-                AudioCdSource cdSource = source as AudioCdSource;
-                if(cdSource.Disk.Udi == args.Udi) {
-                    SourceManager.SetActiveSource(LibrarySource.Instance);
-                }
-            }
-        }
-
-        private void OnAudioCdCoreUpdated(object o, EventArgs args)
-        {
-            Source source = SourceManager.ActiveSource;
-            if(source == null) {
-                return;
-            }
-            
-            playlistView.QueueDraw();
-       
-            if(source is AudioCdSource && playlistModel.FirstTrack != null 
-                && playlistModel.FirstTrack.GetType() == typeof(AudioCdTrackInfo)) {
-                AudioCdSource cdSource = source as AudioCdSource;
-                AudioCdTrackInfo track = playlistModel.FirstTrack as AudioCdTrackInfo;
-               
-                if(cdSource.Disk.DeviceNode == track.Device) { 
-                    (gxml["ViewNameLabel"] as Label).Markup = "<b>" + 
-                        GLib.Markup.EscapeText(cdSource.Disk.Title) + "</b>";
-                    UpdateAudioCdStatus(cdSource.Disk);
-                }
-            }
-        }
-        
         private void OnDapSaveFinished(object o, EventArgs args)
         {
             if(SourceManager.ActiveSource is DapSource
@@ -1801,7 +1671,7 @@ namespace Banshee
                 playlistView.Selection.SelectPath(path);
         }
      
-        private void OnAudioCdRipperTrackRipped(object o, HaveTrackInfoArgs args)
+        /*private void OnAudioCdRipperTrackRipped(object o, HaveTrackInfoArgs args)
         {
             if(SourceManager.ActiveSource is LibrarySource) {
                 ThreadAssist.ProxyToMain(delegate {
@@ -1812,7 +1682,7 @@ namespace Banshee
                     playlistModel.AddTrack(args.TrackInfo);
                 });
             }
-        }
+        }*/
         
         private void EjectSource(Source source)
         {
@@ -1824,7 +1694,9 @@ namespace Banshee
                         }
                     }
                     
-                    source.Eject();
+                    if(!source.Eject()) {
+                        return;
+                    }
                     
                     if(source == SourceManager.ActiveSource) {
                         SourceManager.SetActiveSource(LibrarySource.Instance);
@@ -1843,22 +1715,24 @@ namespace Banshee
             try {
                 TrackInfo ti = new LibraryTrackInfo(args.FileName);
                 args.ReturnMessage = String.Format("{0} - {1}", ti.Artist, ti.Title);
-                if(SourceManager.ActiveSource is LibrarySource) {
-                    ThreadAssist.ProxyToMain(delegate {
-                        if(searchEntry.IsQueryAvailable && !DoesTrackMatchSearch(ti)) {
-                            return;
-                        }
-                        
-                        playlistModel.AddTrack(ti);
-                    });
-                }
-            } catch(Entagged.Audioformats.Exceptions.CannotReadException) {
-                Console.WriteLine(Catalog.GetString("Cannot Import") + ": {0}", args.FileName);
-                args.ReturnMessage = Catalog.GetString("Scanning") + "...";
             } catch(Exception e) {
-                Console.WriteLine(Catalog.GetString("Cannot Import: {0} ({1}, {2})"), 
-                    args.FileName, e.GetType(), e.Message);
                 args.ReturnMessage = Catalog.GetString("Scanning") + "...";
+                
+                switch(Path.GetExtension(args.FileName)) {
+                    case ".m3u":
+                    case ".jpg":
+                    case ".jpeg":
+                    case ".png":
+                    case ".bmp":
+                    case ".gif":
+                        return;
+                }
+                
+                if(e is ApplicationException) {
+                    return;
+                }
+            
+                Console.WriteLine(Catalog.GetString("Cannot Import: {0} ({1})"), args.FileName, e.GetType());
             }
         }
         
@@ -2010,7 +1884,6 @@ namespace Banshee
                 PlayerEngineCore.Open(new Uri(address));
                 PlayerEngineCore.Play();
             } catch(Exception) {
-                Console.WriteLine("BAOSJFDL");
             }   
         }
         
@@ -2023,29 +1896,8 @@ namespace Banshee
         
         private void OnImportCDAction(object o, EventArgs args)
         {
-            ArrayList list = new ArrayList();
-            
-            foreach(object [] node in playlistModel) {
-                if(node[0] is AudioCdTrackInfo && ((AudioCdTrackInfo)node[0]).CanRip) {
-                    list.Add(node[0] as AudioCdTrackInfo);
-                }
-            }
-            
-            if(list.Count > 0) {
-                AudioCdRipper ripper = new AudioCdRipper();
-                ripper.HaveTrackInfo += OnAudioCdRipperTrackRipped;
-                foreach(AudioCdTrackInfo track in list) {
-                    ripper.QueueTrack(track);
-                }
-                ripper.Start();
-            } else {
-                HigMessageDialog dialog = new HigMessageDialog(WindowPlayer, DialogFlags.Modal, 
-                    MessageType.Info, ButtonsType.Ok, 
-                    Catalog.GetString("Invalid Selection"),
-                    Catalog.GetString("You must select at least one track to import.")
-                );
-                dialog.Run();
-                dialog.Destroy();
+            if(SourceManager.ActiveSource is AudioCdSource) {
+                ((IImportSource)SourceManager.ActiveSource).Import();
             }
         }
         
@@ -2245,6 +2097,20 @@ namespace Banshee
             PlayerEngineCore.Position -= SkipDelta;
         }
         
+        private void OnSeekToAction(object o, EventArgs args)
+        {
+            Banshee.Gui.SeekDialog dialog = new Banshee.Gui.SeekDialog();
+            dialog.Dialog.Show();
+            dialog.Dialog.Response += delegate {
+                dialog.Destroy();
+            };
+        }
+        
+        private void OnRestartSongAction(object o, EventArgs args)
+        {
+            PlayerEngineCore.Position = 0;
+        }
+        
         private void SetRepeatMode(RepeatMode mode)
         {
             playlistModel.Repeat = mode;
